/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package pedviz.loader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Vector;

import pedviz.graph.Edge;
import pedviz.graph.Graph;
import pedviz.graph.Node;

/**
 * This Loader class allow you to load a graph from a csv-file. If you write
 * your own Loader classes, this class is a good template.
 * 
 * @author Lukas Forer
 * @version 0.1
 */
public class CsvGraphLoader extends Loader {
    private String filename;

    private String separator;

    private String[] colStrings;

    private ArrayList<String> attributes;

    private HashMap<String, Integer> types;

    public static final int INTEGER = 0;

    public static final int DOUBLE = 1;

    /**
     * Constructs a CsvGraphLoader
     * 
     * @param filename
     *                filename
     * @param separator
     *                Character which separates the values
     */
    public CsvGraphLoader(String filename, String separator) {
	this.filename = filename;
	this.separator = separator;
	this.attributes = new ArrayList<String>();
	this.types = new HashMap<String, Integer>();
    }

    /**
     * Sets the type for the given column.
     * 
     * @param column
     *                the name of the column.
     * @param type
     *                Type (STRING(default)/INTEGER/DOUBLE)
     */
    public void setColumnType(String column, int type) {
	types.put(column, type);
    }

    @Override
    public void load(Graph graph) {
	int colId = -1;
	int colIdMom = -1;
	int colIdDad = -1;

	BufferedReader reader = null;
	FileInputStream fis = null;

	ArrayList<Object[]> myEdges = new ArrayList<Object[]>();

	try {
	    File f = new File(filename);
	    graph.setName(f.getName());
	    fis = new FileInputStream(f);
	    reader = new BufferedReader(new InputStreamReader(fis));

	    String row = reader.readLine();
	    colStrings = row.split(separator);

	    for (int i = 0; i < colStrings.length; i++) {
		// Id
		String colString = colStrings[i].toUpperCase();
		if (colString.equals(getIdColumn().toUpperCase()))
		    colId = i;

		// Mom
		if (colString.equals(getMomColumn().toUpperCase()))
		    colIdMom = i;

		// Dad
		if (colString.equals(getDadColumn().toUpperCase()))
		    colIdDad = i;
		attributes.add(colString);
	    }

	    while ((row = reader.readLine()) != null) {
		String[] columns = row.split(separator);
		Object id = null;
		Object idMom = null;
		Object idDad = null;
		/*
		 * try { id = Integer.parseInt(columns[colId]); if
		 * (!columns[colIdMom].equals("")) idMom =
		 * Integer.parseInt(columns[colIdMom]);
		 * 
		 * if (!columns[colIdDad].equals("")) idDad =
		 * Integer.parseInt(columns[colIdDad]); } catch (Exception e) {
		 */
		id = columns[colId];
		if (!columns[colIdMom].equals(""))
		    idMom = columns[colIdMom];

		if (!columns[colIdDad].equals(""))
		    idDad = columns[colIdDad];
		// }

		Node node = new Node(id);
		node.setIdDad(idDad);
		node.setIdMom(idMom);
		myEdges.add(new Object[] { idDad, id });
		myEdges.add(new Object[] { idMom, id });
		graph.addNode(node);

		for (int i = 0; i < columns.length; i++) {
		    String value = columns[i];
		    if (i == colIdMom) {
			node.setUserData(colStrings[i], idMom);
		    } else if (i == colIdDad) {
			node.setUserData(colStrings[i], idDad);
		    } else {
			Integer type = types.get(colStrings[i]);
			if (type != null) {
			    switch (type) {
			    case INTEGER:
				node.setUserData(colStrings[i], Integer
					.parseInt(value));
				break;
			    case DOUBLE:
				node.setUserData(colStrings[i], Double
					.parseDouble(value));
				break;
			    default:
				node.setUserData(colStrings[i], value);
			    }
			} else {
			    node.setUserData(colStrings[i], value);
			}
		    }
		}
	    }

	    for (Object[] edge : myEdges) {
		Node start = graph.getNode(edge[0]);
		Node end = graph.getNode(edge[1]);
		if (start != null && end != null) {
		    Edge edge2 = new Edge(start, end);
		    graph.addEdge(edge2);
		}
	    }

	} catch (IOException e) {
	    System.out.println(e.getMessage());
	} finally {
	    if (reader != null) {
		try {
		    reader.close();
		} catch (IOException ioe) {
		    System.out.println(ioe.getMessage());
		}
	    }
	}
    }

    /**
     * Returns a collection of all attributes.
     * 
     * @return a collection of all attributes.
     */
    public ArrayList<String> getAttributes() {
	return attributes;
    }

    @Override
    public Vector<String> getColumns() {
	Vector<String> colStrings = new Vector<String>();

	BufferedReader reader = null;
	FileInputStream fis = null;

	try {
	    fis = new FileInputStream(new File(filename));
	    reader = new BufferedReader(new InputStreamReader(fis));

	    String row = reader.readLine();
	    String[] tiles = row.split(separator);
	    for (String tile : tiles) {
		colStrings.add(tile);
	    }
	} catch (Exception e) {
	    System.out.println("......." + e.getMessage());
	} finally {
	    if (reader != null) {
		try {
		    reader.close();
		} catch (IOException ioe) {
		    System.out.println(ioe.getMessage());
		}
	    }
	}
	return colStrings;
    }

}
