/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package pedviz.loader;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Vector;

import pedviz.graph.Edge;
import pedviz.graph.Graph;
import pedviz.graph.Node;

/**
 * This Loader class allow you to load a graph directly from a database. It's
 * possible to load the whole table or to define your own querie.
 * 
 * @author Lukas Forer
 * @version 0.1
 */
public class DatabaseGraphLoader extends Loader {
    private String table;

    private String query = "";

    private Vector<String> columns = null;

    private ArrayList<String> attributes;

    private Database database;

    /**
     * Constructs a new DatabaseGraphLoader with the given connection and the
     * table name.
     * 
     * @param connection
     *                Connection object
     * @param table
     *                tablename
     */
    public DatabaseGraphLoader(Database database, String table) {
	this.database = database;
	this.table = table;
	this.attributes = new ArrayList<String>();
    }

    /**
     * Sets a query.
     * 
     * @param query
     */
    public void setQuery(String query) {
	this.query = query;
    }

    @Override
    public void load(Graph graph) {
	ArrayList<Object[]> myEdges = new ArrayList<Object[]>();

	if (query.equals(""))
	    query = "SELECT * FROM " + table;

	try {
	    ResultSet rs = database.executeQuerie(query);

	    rs.beforeFirst();

	    while (rs.next()) {
		// Id
		Object id = rs.getObject(getIdColumn());
		Node node = new Node(id);
		graph.addNode(node);

		// Mom
		Object idMom = rs.getObject(getMomColumn());
		myEdges.add(new Object[] { idMom, id });
		node.setIdMom(idMom);

		// Dad
		Object idDad = rs.getObject(getDadColumn());
		myEdges.add(new Object[] { idDad, id });
		node.setIdDad(idDad);

		// save all attributes in userdata
		for (int i = 1; i <= rs.getMetaData().getColumnCount(); i++) {
		    node.setUserData(rs.getMetaData().getColumnName(i), rs
			    .getObject(i));
		    if (rs.isFirst())
			attributes.add(rs.getMetaData().getColumnName(i));
		}

	    }

	    for (Object[] edge : myEdges) {
		Node start = graph.getNode(edge[0]);
		Node end = graph.getNode(edge[1]);

		if (start != null && end != null)
		    graph.addEdge(new Edge(start, end));
	    }

	} catch (Exception e) {
	    System.out.println(e);
	}
    }

    @Override
    public Vector<String> getColumns() {
	if (columns == null) {
	    try {
		DatabaseMetaData meta = database.getConnection().getMetaData();
		ResultSet rsColumns = meta.getColumns(null, "%", table, "%");
		columns = new Vector<String>();
		rsColumns.beforeFirst();
		while (rsColumns.next()) {
		    columns.add(rsColumns.getString("COLUMN_NAME"));
		}
	    } catch (Exception e) {
		System.out.println(e);
	    }
	}
	return columns;
    }

    /**
     * Returns a collection of all attributes.
     * 
     * @return a collection of all attributes.
     */
    public ArrayList<String> getAttributes() {
	return attributes;
    }
}
